import java.util.Random;

/**
 * ObjectListElem objects are elemets of a ObjectList list.
 * 
 * @author Silvia Santini, bcyrill 
 * Informatik II - SS2007, 
 * Uebungsserie 6
 */
class ObjectListElem {
	Object value;

	ObjectListElem next; // reference to the next element of the list

	/**
	 * IntListElem constructor
	 * 
	 * @param v
	 *            value of the element being created
	 * @param e
	 *            reference to the next element in the list
	 */
	public ObjectListElem(Object v, ObjectListElem e) {
		value = v;
		next = e;
	}
} // end class IntListElement

/**
 * The class IntList provides methods to create and use lists of Objects.
 * 
 * @author Silvia Santini, bcyrill <br>
 *         Informatik II - SS2007 <br>
 *         Uebungsserie 5, Aufgaben 1-3 <br>
 */
public class ObjectList {
	//	 reference to the first element of the list
	private ObjectListElem first; 

	/**
	 * Creates an empty list
	 */
	public ObjectList() {
		first = null;
	}

	/**
	 * The method empty() tests whether a list is empty or not
	 * 
	 * @return <code>true</code> if the list is empty, <code>false</code> otherwise
	 */
	public boolean empty() {
		return first == null;
	}

	/**
	 * Adds an element at the top of the list
	 * 
	 * @param i
	 *            element to add
	 */
	public void addFirst(Object i) {
		// Listen von Objekten sollen nicht als ein Objekt interpretiert werden.
		// Anzufuegende Liste von Hinten nach Vorne abschneiden und bei der
		// Neuen am Anfang einfuegen. -> Reihenfolge
		if (i instanceof ObjectList) {
			ObjectList tmp = (ObjectList) i;
			try {
				while (!tmp.empty()) {
					this.addFirst(tmp.removeLast());
				}
			} catch (Exception e) {
				System.out.println("Internal Error");
			}
		} else {
			first = new ObjectListElem(i, first);
		}
	}

	/**
	 * Adds the element i to the end of the list
	 * 
	 * @param i
	 *            value of the element to add
	 */
	public void addLast(Object i) {
		ObjectListElem elem = new ObjectListElem(i, null);
		if (first == null) {
			first = elem;
		} else {
			ObjectListElem tmp = first;
			while (tmp.next != null) {
				tmp = tmp.next;
			}
			tmp.next = elem;
		}
	}

	/**
	 * Removes and returns the first element of the list
	 * 
	 * @return the first element of the list
	 */
	public Object removeFirst() throws Exception {
		if (first == null) {
			throw new Exception("removeFirst() called on an empty list.");
		}
		Object i = first.value;
		first = first.next;
		return i;
	}

	/**
	 * Removes and returns the last element of the list
	 * 
	 * @return the last element of the list
	 */
	public Object removeLast() throws Exception {
		Object i = null;
		if (first == null) {
			// list is empty
			throw new Exception("removeLast() called on an empty list.");

		} else if (first.next == null) {
			// list only has the first element
			i = first.value;
			first = null;
		} else {
			// list has more than one element
			ObjectListElem tmp = first;
			while (tmp.next.next != null) {
				tmp = tmp.next;
			}
			i = tmp.next.value;
			tmp.next = null;
		}
		return i;
	}

	/**
	 * Returns a string containing the elements of a list (bungsserie 6,
	 * Aufgabe 2b)
	 * 
	 * @return a string containing the elements
	 */
	public String toString() {
		String str = "(";
		ObjectListElem elem = first;
		while (elem != null) {
			str += elem.value.toString();
			elem = elem.next;
			if (elem != null)
				str += ",";
		}
		str += ")";
		return str;
	}
	/**
	 * Returns a string containing the elements and type of a list (bungsserie 6,
	 * Aufgabe 2b)
	 * 
	 * @return a string containing the elements and type
	 */
	public String toVerboseString() {
		String str = "(";
		ObjectListElem elem = first;
		while (elem != null) {
			str = str + elem.value.toString();
			if (elem.value instanceof Boolean) {
				str += "(Boolean)";
			} else if (elem.value instanceof Integer) {
				str += "(Integer)";
			} else if (elem.value instanceof String) {
				str += "(String)";
			} else if (elem.value instanceof Character) {
				str += "(Character)";
			} else {
				str += "(Unknown)";
			}
			elem = elem.next;
			if (elem != null) {
				str += ",";
			}
		}
		str += ")";
		return str;
	}

	/**
	 * Adds n (pseudo-)random int elements to the top of the list
	 * 
	 * @param n
	 *            number of elements to add
	 */
	public void addRandomElements(int n) {
		Random rnd = new Random();
		for (int i = 0; i < n; i++) {
			Object tmp = new Integer(rnd.nextInt(200));
			addFirst(tmp);
		}
	}

	/**
	 * Checks if every element of the list implement Comparable
	 * 
	 * @return <code>true</code> if every element implements 
	 * 			Comparable <code>false</code> otherwise
	 */
	private boolean checkComparable() {
		ObjectListElem elem = first;
		while (elem != null) {
			if (!(elem.value instanceof Comparable)) {
				return false;
			}
			elem = elem.next;
		}
		return true;
	}

	/**
	 * Sorts a list of objects.
	 */
	public void sortSelf() {
		ObjectList sorted = sort();
		first = sorted.first;
	}

	/**
	 * Ceates a sorted list from an unsorted one. After being sorted the
	 * original list is empty.
	 * 
	 * @return reference to the (new) sorted list
	 */
	public ObjectList sort() {
		if (!this.checkComparable()) {
			System.out.println("Not Comparable");
			return this;
		} else {
			ObjectList list = new ObjectList();
			try {
				while (!this.empty()) {
					list.insertSorted(this.removeLast());
				}
			} catch (Exception e) {
				System.out.println("Internal Error in method sort().");
				e.printStackTrace();
				System.exit(1);
			}
			return list;
		}
	}

	/**
	 * Insert the element i in a ordered list in the correct position. The
	 * elements in the list are supposed to be in ascending order
	 * 
	 * @param i
	 *            element to add
	 */
	private void insertSorted(Object i) {
		if (first == null) {
			// initial list is empty
			first = new ObjectListElem(i, null);
		} else {
			// initial list has at least one element
			if (((Comparable) (i)).compareTo((Comparable) (first.value)) < 0) {
				// add as first
				addFirst(i);
			} else {
				// add someplace after the first
				ObjectListElem elem = first;
				while (elem.next != null) {
					if (((Comparable) (i))
							.compareTo((Comparable) (elem.next.value)) < 0) {
						// insert between e and e.next
						elem.next = new ObjectListElem(i, elem.next);
						return;
					}
					elem = elem.next;
				} // end while()
				// end of list reached and not yet inserted: add last
				addLast(i);
			}
		}
	} // end insertSorted()

} // end class IntList

